/****************************************************************************
* VCGLib                                                            o o     *
* Visual and Computer Graphics Library                            o     o   *
*                                                                _   O  _   *
* Copyright(C) 2004-2016                                           \/)\/    *
* Visual Computing Lab                                            /\/|      *
* ISTI - Italian National Research Council                           |      *
*                                                                    \      *
* All rights reserved.                                                      *
*                                                                           *
* This program is free software; you can redistribute it and/or modify      *
* it under the terms of the GNU General Public License as published by      *
* the Free Software Foundation; either version 2 of the License, or         *
* (at your option) any later version.                                       *
*                                                                           *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
* GNU General Public License (http://www.gnu.org/licenses/gpl.txt)          *
* for more details.                                                         *
*                                                                           *
****************************************************************************/
/*! \file trimesh_base.cpp
\ingroup code_sample

\brief the minimal example of using the lib

This file contain a minimal example of the library, showing how to load a mesh and how to compute per vertex normals on it.

*/

#include<vcg/complex/complex.h>
#include<wrap/io_trimesh/import_off.h>
#include<vcg/complex/algorithms/clean.h>

class MyVertex; class MyEdge; class MyFace;
struct MyUsedTypes : public vcg::UsedTypes<vcg::Use<MyVertex>   ::AsVertexType,
                                           vcg::Use<MyEdge>     ::AsEdgeType,
                                           vcg::Use<MyFace>     ::AsFaceType>{};

class MyVertex  : public vcg::Vertex< MyUsedTypes, vcg::vertex::Coord3f, vcg::vertex::Normal3f, vcg::vertex::BitFlags  >{};
class MyFace    : public vcg::Face<   MyUsedTypes, vcg::face::FFAdj,  vcg::face::VertexRef, vcg::face::BitFlags > {};
class MyEdge    : public vcg::Edge<   MyUsedTypes> {};

class MyMesh    : public vcg::tri::TriMesh< std::vector<MyVertex>, std::vector<MyFace> , std::vector<MyEdge>  > {};
using namespace vcg;

int main( int argc, char **argv )
{
  if(argc<2)
  {
    printf("Usage trimesh_base <meshfilename.obj>\n");
    return -1;
  }
  MyMesh m;

  if(vcg::tri::io::ImporterOFF<MyMesh>::Open(m,argv[1])!=vcg::tri::io::ImporterOFF<MyMesh>::NoError)
  {
    printf("Error reading file  %s\n",argv[1]);
    exit(0);
  }

  printf("Input mesh  vn:%i fn:%i\n",m.VN(),m.FN());  

  MyMesh::PerVertexAttributeHandle<int> arityHandle = 
      tri::Allocator<MyMesh>::GetPerVertexAttribute<int>(m,"arity");
  
  tri::ForEachVertex(m, [&](MyVertex &v){
               arityHandle[v]=0;
  });
  
  tri::ForEachFace(m, [&](MyFace &f){
    for(int i=0;i<3;++i)      
      ++arityHandle[f.V(i)];
  });
  
  int maxArity=0;
  tri::ForEachVertex(m, [&](MyVertex &v){
               maxArity=std::max(maxArity,arityHandle[v]);
  });
  
  printf("MaxArity = %i\n",maxArity );
  
  // TODO: color the mesh according to the arity of the vertexes. 
  // 1) add a vcg::vertex::Color4b component to the MyVertex type definition
  // 2) access the color member of a vertex vi->C() and modify it according to a color scale between 0 and max arity 
  // 3) use the SetColorRamp function of the Color4b 
  // 4) use the io file masks to explicitly specify that you want to save per-vertex color 
  //    vcg::tri::io::ExporterPLY<MyMesh>::Save(m,"out.ply", vcg::tri::io::Mask::IOM_VERTCOLOR);
  //    (not all the formats support this info, ply is ok http://www.vcglib.net/fileformat.html ) 
  //    
  //
  // 5) eventually, as an extra, try to compute the discrete Gaussian curvature over the vertexes
  //    by computing for each vertex the angle excess/defect.
  //    Useful functions for this purpose:  
  //      - Angle(Point3f,Point3f) that returns the angle between two 3D vectors
  //      - V0(i) V1(i) V2(i) members of the vertex that returns the (i+k)%3 th vertex of a face 
  //            so that ith edge vector is:
  //                    vi->V1(i) - vi->V0(i)
    
  
  
  return 0;
}





